<?php
/*
 * File name: GoogleFormAPIController.php
 * Last modified: 2021.11.01 at 22:25:44
 * Author: SmarterVision - https://codecanyon.net/user/smartervision
 * Copyright (c) 2021
 */

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use App\Repositories\AppointmentRepository;
use App\Notifications\NewAppointment;
use Illuminate\Validation\ValidationException;
use App\Repositories\DoctorRepository;
use App\Repositories\AddressRepository;
use App\Repositories\DoctorPatientsRepository;
use App\Repositories\CouponRepository;
use App\Repositories\UserRepository;
use App\Repositories\AppointmentStatusRepository;
use App\Repositories\PatientRepository;
use Prettus\Repository\Exceptions\RepositoryException;
use Prettus\Validator\Exceptions\ValidatorException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Notification;
use Exception;

/**
 * Class GoogleFormAPIController
 * @package App\Http\Controllers\API
 */
class GoogleFormAPIController extends Controller
{
    /** @var  AppointmentRepository */
    private $appointmentRepository;
    /**
     * @var DoctorRepository
     */
    private $doctorRepository;
    /**
     * @var UserRepository
     */
    private $userRepository;
    /**
     * @var PatientRepository
     */
    private $patientRepository;
    /**
     * @var AddressRepository
     */
    private $addressRepository;
    /**
     * @var DoctorPAtientsRepository
     */
    private $doctorPatientsRepository;
    /**
     * @var AppointmentStatusRepository
     */
    private $appointmentStatusRepository;
    /**
     * @var CouponRepository
     */
    private $couponRepository;

    public function __construct(
        AppointmentRepository $appointmentRepo,
        DoctorRepository $doctorRepository,
        UserRepository $userRepo,
        PatientRepository $patientRepository,
        AddressRepository $addressRepository,
        DoctorPatientsRepository $doctorPatientsRepository,
        AppointmentStatusRepository $appointmentStatusRepo,
        CouponRepository $couponRepository,
    ) {
        parent::__construct();
        $this->appointmentRepository = $appointmentRepo;
        $this->doctorRepository = $doctorRepository;
        $this->userRepository = $userRepo;
        $this->patientRepository = $patientRepository;
        $this->addressRepository = $addressRepository;
        $this->doctorPatientsRepository = $doctorPatientsRepository;
        $this->appointmentStatusRepository = $appointmentStatusRepo;
        $this->couponRepository = $couponRepository;
    }

    /**
     * Display a listing of the Google Forms.
     * GET|HEAD /googleforms
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function index(Request $request): JsonResponse
    {
        return response()->json(['message' => 'hello']);
    }

    /**
     * Display the specified Google Form.
     * GET|HEAD /googleforms/{id}
     *
     * @param int $id
     * @param Request $request
     * @return JsonResponse
     */
    /**
     * Decodes JSON data if it's in JSON format or returns the string as is.
     * 
     * @param string $data
     * @return string
     */
    private function decodeName($data)
    {
        $decoded = json_decode($data, true);
        if (json_last_error() === JSON_ERROR_NONE) {
            return $decoded['en']; // Assuming English names are stored under 'en'
        }
        return $data; // Return the original data if it's not JSON
    }

    public function show($id, Request $request): JsonResponse
    {

        $url = "https://script.google.com/macros/s/AKfycbxH8Fv2gXb3OOJo_2RFh84iL2LjQFozSlYiBxEEVvB0lc4FoHLOitNhviPLFGS0O1Sp/exec";
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);

        $data = curl_exec($curl);
        if ($data === false) {
            curl_close($curl);
            return response()->json(['error' => 'Failed to fetch data'], 500);
        }

        curl_close($curl);
        $decodedData = json_decode($data, true);

        if (is_null($decodedData) || !is_array($decodedData)) {
            return response()->json(['error' => 'Invalid JSON received', 'raw' => $data], 500);
        }

        foreach ($decodedData as $appointmentData) {
            $doctorName = $appointmentData['doctor'] ?? null;

            if (!$doctorName) {
                return response()->json(['error' => 'Doctor name missing in appointment data', 'appointmentData' => $appointmentData], 500);
            }

            $doctors = $this->doctorRepository->all();
            $doctor = $doctors->filter(function ($doc) use ($doctorName) {
                if (!$doc || !isset($doc->name)) return false;
                $docName = $this->decodeName($doc->name);
                return $docName === $doctorName;
            })->first();

            if (!$doctor) {
                return response()->json(['error' => 'Doctor not found', 'doctorName' => $doctorName], 500);
            }

            $doctor->user = $this->userRepository->find($doctor->user_id);

            if (!$doctor->user) {
                return response()->json(['error' => 'No user found for the doctor', 'doctor' => $doctor], 500);
            }

            $clinic = $doctor->clinic;
            if (!$clinic) {
                return response()->json(['error' => 'No clinic found for the doctor', 'doctor' => $doctor], 500);
            }

            $patientMobile = $appointmentData['patientid'] ?? null;
            if (!$patientMobile) {
                return response()->json(['error' => 'Patient mobile number missing in appointment data', 'appointmentData' => $appointmentData], 500);
            }

            $patient = $this->patientRepository->findWhere(['mobile_number' => $patientMobile])->first();
            if (!$patient) {
                return response()->json(['error' => 'Patient not found for mobile number', 'patientMobile' => $patientMobile], 500);
            }

            $taxes = $clinic->taxes;
            $appointmentAt = $appointmentData['appointment'] ?? null;
            $startAt = $appointmentData['start_at'] ?? null;
            $endsAt = $appointmentData['ends_at'] ?? null;

            $input = [
                'address' => $clinic->address ?? null,
                'clinic' => $clinic,
                'taxes' => $taxes,
                'doctor' => $doctor,
                'patient' => $patient,
                'appointment_status_id' => $this->appointmentStatusRepository->find(1)->id,
                'appointment_at' => $appointmentAt,
                'start_at' => $startAt,
                'ends_at' => $endsAt,
            ];

            if (isset($appointmentData['coupon_id'])) {
                $input['coupon'] = $this->couponRepository->find($appointmentData['coupon_id']);
            }

            $this->doctorPatientsRepository->create(['doctor_id' => $doctor->id, 'patient_id' => $patient->id]);
            $appointment = $this->appointmentRepository->create($input);

            // Notification::send($doctor->user, new NewAppointment($appointment));
        }

        return response()->json(['message' => 'Appointments created successfully']);
    }






    /**
     * Remove the specified Google Form from storage.
     * DELETE /googleforms/{id}
     *
     * @param int $id
     * @return JsonResponse
     */
    public function destroy($id): JsonResponse
    {
        return response()->json(['message' => 'hello']);
    }
}
