<?php
/*
 * File name: AppointmentController.php
 * Last modified: 2021.11.01 at 22:25:44
 * Author: SmarterVision - https://codecanyon.net/user/smartervision
 * Copyright (c) 2021
 */

namespace App\Http\Controllers;

use App\Criteria\Addresses\AddressesOfUserCriteria;
use App\Criteria\Appointments\AppointmentsOfPatientCriteria;
use App\DataTables\AppointmentDataTable;
use App\Events\AppointmentChangedEvent;
use App\Notifications\NewAppointment;
use App\Http\Requests\CreateAppointmentRequest;
use App\Http\Requests\UpdateAppointmentRequest;
use App\Models\Appointment;
use App\Models\Patient;
use App\Notifications\StatusChangedAppointment;
use App\Repositories\AddressRepository;
use App\Repositories\AppointmentRepository;
use App\Repositories\AppointmentStatusRepository;
use App\Repositories\CouponRepository;
use App\Repositories\CustomFieldRepository;
use App\Repositories\ClinicRepository;
use App\Repositories\DoctorRepository;
use App\Repositories\NotificationRepository;
use App\Repositories\OptionRepository;
use Illuminate\Http\Request;
use App\Repositories\PatientRepository;
use App\Repositories\DoctorPatientsRepository;
use App\Repositories\PaymentRepository;
use App\Repositories\PaymentStatusRepository;
use App\Repositories\TaxRepository;
use App\Repositories\UserRepository;
use Illuminate\Foundation\Http\FormRequest;
use Flash;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Routing\Redirector;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Facades\Response;
use Illuminate\View\View;
use Prettus\Repository\Exceptions\RepositoryException;
use Prettus\Validator\Exceptions\ValidatorException;

class AppointmentController extends Controller
{
    /** @var  AppointmentRepository */
    private $appointmentRepository;

    /**
     * @var CustomFieldRepository
     */
    private $customFieldRepository;

    /**
     * @var UserRepository
     */
    private $userRepository;
    /**
     * @var AppointmentStatusRepository
     */
    private $appointmentStatusRepository;
    /**
     * @var PaymentRepository
     */
    private $paymentRepository;
    /**
     * @var NotificationRepository
     */
    private $notificationRepository;
    /**
     * @var AddressRepository
     */
    private $addressRepository;
    /**
     * @var TaxRepository
     */
    private $taxRepository;
    /**
     * @var DoctorRepository
     */
    private $doctorRepository;
    /**
     * @var ClinicRepository
     */
    private $clinicRepository;
    /**
     * @var CouponRepository
     */
    private $couponRepository;
    /**
     * @var PatientRepository
     */
    private $patientRepository;
    /**
     * @var DoctorPatientsRepository
     */
    private $doctorPatientsRepository;
    /**
     * @var PaymentStatusRepository
     */
    private $paymentStatusRepository;

    public function __construct(
        AppointmentRepository $appointmentRepo,
        CustomFieldRepository $customFieldRepo,
        UserRepository $userRepo,
        AppointmentStatusRepository                 $appointmentStatusRepo,
        NotificationRepository $notificationRepo,
        PaymentRepository $paymentRepo,
        AddressRepository $addressRepository,
        TaxRepository $taxRepository,
        DoctorRepository $doctorRepository,
        ClinicRepository $clinicRepository,
        CouponRepository $couponRepository,
        PatientRepository $patientRepository,
        PaymentStatusRepository $paymentStatusRepository,
        DoctorPatientsRepository $doctorPatientsRepo
    ) {
        parent::__construct();
        $this->appointmentRepository = $appointmentRepo;
        $this->customFieldRepository = $customFieldRepo;
        $this->userRepository = $userRepo;
        $this->appointmentStatusRepository = $appointmentStatusRepo;
        $this->notificationRepository = $notificationRepo;
        $this->paymentRepository = $paymentRepo;
        $this->addressRepository = $addressRepository;
        $this->taxRepository = $taxRepository;
        $this->doctorRepository = $doctorRepository;
        $this->clinicRepository = $clinicRepository;
        $this->couponRepository = $couponRepository;
        $this->patientRepository = $patientRepository;
        $this->paymentStatusRepository = $paymentStatusRepository;
        $this->doctorPatientsRepository = $doctorPatientsRepo;
    }

    /**
     * Display a listing of the Appointment.
     *
     * @param AppointmentDataTable $appointmentDataTable
     * @return Response
     */
    public function index(AppointmentDataTable $appointmentDataTable)
    {
        return $appointmentDataTable->render('appointments.index');
    }

    /**
     * Show the form for creating a new Appointment.
     *
     * @return Application|Factory|Response|View
     */
    public function create()
    {
        $appointments = $this->appointmentRepository->pluck('id', 'id');
        $user = $this->userRepository->pluck('name', 'id');
        $paymentStatuses = $this->paymentStatusRepository->pluck('status', 'id');
        $patient = $this->patientRepository->pluck('first_name', 'id');
        $doctor = $this->doctorRepository->pluck('name', 'id');
        $appointmentStatus = $this->appointmentStatusRepository->orderBy('order')->pluck('status', 'id');
        $addresses = $this->addressRepository->pluck('address', 'id');

        $hasCustomField = in_array($this->appointmentRepository->model(), setting('custom_field_models', []));
        if ($hasCustomField) {
            $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->appointmentRepository->model());
            $html = generateCustomField($customFields);
        }

        return view('appointments.create')->with('appointments', $appointments)
            ->with("customFields", isset($html) ? $html : false)
            ->with("appointmentStatus", $appointmentStatus)
            ->with("addresses", $addresses)
            ->with("paymentStatuses", $paymentStatuses)
            ->with("user", $user)
            ->with("patient", $patient)
            ->with("doctor", $doctor);
    }



    /**
     * Store a newly created Appointmnet in storage.
     *
     * @param CreateAppointmentRequest $request
     *
     * @return Response
     */
    public function store(CreateAppointmentRequest $request)
    {
        // Extract all input data from the request
        
        $input = $request->all();

        // Check if doctor ID is provided and valid
        $doctorId = $request->input('doctor_id');
        if (!$doctorId) {
            Flash::error('Doctor ID is required.');
            return back()->withInput();
        }

        // Attempt to find the doctor with the provided ID
        $doctor = $this->doctorRepository->find($doctorId);
        if (!$doctor) {
            Flash::error('Doctor not found.');
            return back()->withInput();
        }

        // Assign associated user to the doctor
        $doctor->user = $this->userRepository->find($doctor->user_id);
        $user = $this->userRepository->find($input['user_id']);
        $clinic = $doctor->clinic;

        // Validate patient information
        $patientId = $request->input('patient_id');
        if (!isset($patientId)) {
            Flash::error('Patient information is required.');
            return back()->withInput();
        }
        $patient = $this->patientRepository->find($patientId);

        // Process address information
        $taxes = $clinic->taxes;
        if (isset($input['address']) && $input['address']['address'] && $input['address']['longitude'] && $input['address']['latitude']) {
            $this->validate($request, [
                'address.address' => Address::$rules['address'],
                'address.longitude' => Address::$rules['longitude'],
                'address.latitude' => Address::$rules['latitude'],
            ]);
            $address = $this->addressRepository->updateOrCreate(['address' => $input['address']['address']], $input['address']);
            $input['address'] = $address;
        } else {
            $input['address'] = $clinic->address;
        }

        // Prepare other appointment details
        $input['clinic'] = $clinic;
        $input['taxes'] = $taxes;
        $input['doctor'] = $doctor;
        $input['patient'] = $patient;

        // Create doctor-patient link
        $this->doctorPatientsRepository->create(['doctor_id' =>  $doctor->id, 'patient_id' =>  $patient->id]);
        $input['appointment_status_id'] = $this->appointmentStatusRepository->find(1)->id;

        // Apply coupon if provided
        if (isset($input['coupon_id'])) {
            $input['coupon'] = $this->couponRepository->find($input['coupon_id']);
        }

        // Attempt to create the appointment
        try {
            $appointment = $this->appointmentRepository->create($input);
            Notification::send($doctor->user, new NewAppointment($appointment));
            
            Flash::success(__('lang.saved_successfully', ['operator' => __('lang.appointment')]));
            return redirect(route('appointments.index'));
        } catch (Exception $e) {
            Flash::error($e->getMessage());
            return back()->withInput();
        }
    }







    /**
     * Display the specified Appointment.
     *
     * @param int $id
     *
     * @return Application|Factory|Response|View
     * @throws RepositoryException
     */
    public function show(int $id)
    {
        $id = (int) $id;
        $this->appointmentRepository->pushCriteria(new AppointmentsOfPatientCriteria(auth()->id()));
        $appointment = $this->appointmentRepository->findWithoutFail($id);
        if (empty($appointment)) {
            Flash::error(__('lang.not_found', ['operator' => __('lang.appointment')]));
            return redirect(route('appointments.index'));
        }
        $appointmentStatus = $this->appointmentStatusRepository->orderBy('order')->all();
        return view('appointments.show')->with('appointment', $appointment)->with('appointmentStatuses', $appointmentStatus);
    }

    /**
     * Show the form for editing the specified Appointment.
     *
     * @param int $id
     *
     * @return Application|RedirectResponse|Redirector|Response
     * @throws RepositoryException
     */


    public function edit(int $id)
    {
        $this->appointmentRepository->pushCriteria(new AppointmentsOfPatientCriteria(auth()->id()));
        $appointment = $this->appointmentRepository->findWithoutFail($id);
        //  dd($appointment);
        if (empty($appointment)) {
            Flash::error(__('lang.not_found', ['operator' => __('lang.appointment')]));
            return redirect(route('appointments.index'));
        }

        // Fetch patient details 
        $patient = $this->patientRepository->pluck('first_name', 'id');
        $doctor = $this->doctorRepository->pluck('name', 'id');
        if (!$patient) {
            Flash::error('Patient not found.');
            return back();
        }
        if (!$doctor) {
            Flash::error('Patient not found.');
            return back();
        }
        $user = $this->userRepository->pluck('name', 'id');
        array_push($appointment->fillable, ['address_id', 'payment_status_id']);
        $appointment->address_id = $appointment->address->id;
        $appointmentStatus = $this->appointmentStatusRepository->orderBy('order')->pluck('status', 'id');
        if (!empty($appointment->payment_id)) {
            $appointment->payment_status_id = $appointment->payment->payment_status_id;
            $paymentStatuses = $this->paymentStatusRepository->pluck('status', 'id');
        } else {
            $paymentStatuses = null;
        }
        $addresses = $this->addressRepository->getByCriteria(new AddressesOfUserCriteria($appointment->user_id))->pluck('address', 'id');

        $customFieldsValues = $appointment->customFieldsValues()->with('customField')->get();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->appointmentRepository->model());
        $hasCustomField = in_array($this->appointmentRepository->model(), setting('custom_field_models', []));
        if ($hasCustomField) {
            $html = generateCustomField($customFields, $customFieldsValues);
        }
        return view('appointments.edit')
            ->with('appointment', $appointment)
            ->with('patient', $patient)
            ->with('doctor', $doctor)
            ->with("customFields", isset($html) ? $html : false)
            ->with("appointmentStatus", $appointmentStatus)
            ->with("addresses", $addresses)
            ->with("paymentStatuses", $paymentStatuses)
            ->with("user", $user);
    }


    /**
     * Update the specified Appointment in storage.
     *
     * @param int $id
     * @param UpdateAppointmentRequest $request
     *
     * @return Application|RedirectResponse|Redirector|Response
     */
    public function update(int $id, UpdateAppointmentRequest $request)
    {
        $oldAppointment = $this->appointmentRepository->findWithoutFail($id);
        if (empty($oldAppointment)) {
            Flash::error(__('lang.not_found', ['operator' => __('lang.appointment')]));
            return redirect(route('appointments.index'));
        }
        $input = $request->all();
        $address = $this->addressRepository->findWithoutFail($input['address_id']);
        $input['address'] = $address;
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->appointmentRepository->model());
        try {
            $appointment = $this->appointmentRepository->update($input, $id);
            $user = $appointment->user;
            $doctor = $this->doctorRepository->findWithoutFail($appointment->doctor->id);
            if (isset($input['payment_status_id'])) {
                $this->paymentRepository->update(
                    ['payment_status_id' => $input['payment_status_id']],
                    $appointment->payment_id
                );
                event(new AppointmentChangedEvent($appointment));
            }

            if (isset($input['appointment_status_id']) && $input['appointment_status_id'] != $oldAppointment->appointment_status_id) {
                if ($appointment->appointmentStatus->order < 40) {
                    Notification::send([$user], new StatusChangedAppointment($appointment));
                } else {
                    Notification::send([$doctor->user], new StatusChangedAppointment($appointment));
                }
            }

            foreach (getCustomFieldsValues($customFields, $request) as $value) {
                $appointment->customFieldsValues()
                    ->updateOrCreate(['custom_field_id' => $value['custom_field_id']], $value);
            }
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }
        Flash::success(__('lang.updated_successfully', ['operator' => __('lang.appointment')]));
        return redirect(route('appointments.index'));
    }

    /**
     * Remove the specified Appointment from storage.
     *
     * @param int $id
     *
     * @return Application|RedirectResponse|Redirector|Response
     */
    public function destroy($id)
    {
        if (!config('installer.demo_app')) {
            $this->appointmentRepository->pushCriteria(new AppointmentsOfPatientCriteria(auth()->id()));
            $appointment = $this->appointmentRepository->findWithoutFail($id);

            if (empty($appointment)) {
                Flash::error(__('lang.not_found', ['operator' => __('lang.appointment')]));

                return redirect(route('appointments.index'));
            }

            $this->appointmentRepository->delete($id);

            Flash::success(__('lang.deleted_successfully', ['operator' => __('lang.appointment')]));
        } else {
            Flash::warning('This is only demo app you can\'t change this section ');
        }
        return redirect(route('appointments.index'));
    }

    public function getPatientsByUser($userId)
    {
        try {
            $patients = $this->patientRepository->findWhere(['user_id' => $userId]);
            return response()->json($patients->map(function ($patient) {
                return ['id' => $patient->id, 'name' => $patient->first_name . ' ' . $patient->last_name];
            }));
        } catch (\Exception $e) {
            Log::error("Error fetching patients: " . $e->getMessage());
            return response()->json(['error' => 'Server Error'], 500);
        }
    }
}
