<?php
/*
 * File name: LanguageController.php
 * Last modified: 2021.01.25 at 15:38:34
 * Author: SmarterVision - https://codecanyon.net/user/smartervision
 * Copyright (c) 2021
 */

namespace App\Http\Controllers;

use App\DataTables\LanguageDataTable;
use App\Http\Requests\CreateLanguageRequest;
use App\Http\Requests\UpdateLanguageRequest;
use App\Repositories\LanguageRepository;
use App\Repositories\CustomFieldRepository;
use App\Repositories\UploadRepository;
use Exception;
use Flash;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Response;
use Illuminate\View\View;
use Prettus\Validator\Exceptions\ValidatorException;

class LanguageController extends Controller
{
    /** @var  LanguageRepository */
    private $languageRepository;

    /**
     * @var CustomFieldRepository
     */
    private $customFieldRepository;

    /**
     * @var UploadRepository
     */
    private $uploadRepository;

    public function __construct(LanguageRepository $languageRepo, CustomFieldRepository $customFieldRepo, UploadRepository $uploadRepo)
    {
        parent::__construct();
        $this->languageRepository = $languageRepo;
        $this->customFieldRepository = $customFieldRepo;
        $this->uploadRepository = $uploadRepo;
    }

    /**
     * Display a listing of the Language.
     *
     * @param LanguageDataTable $languageDataTable
     * @return Response
     */
    public function index(LanguageDataTable $languageDataTable)
    {
        return $languageDataTable->render('language.index');
    }

    /**
     * Show the form for creating a new Language.
     *
     * @return Application|Factory|Response|View
     */
    public function create()
    {
        $parentLanguage = $this->languageRepository->pluck('name', 'id');

        $hasCustomField = in_array($this->languageRepository->model(), setting('custom_field_models', []));
        if ($hasCustomField) {
            $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->languageRepository->model());
            $html = generateCustomField($customFields);
        }
        return view('language.create')->with("customFields", isset($html) ? $html : false)->with("parentLanguage", $parentLanguage);
    }

    /**
     * Store a newly created Language in storage.
     *
     * @param CreateLanguageRequest $request
     *
     * @return Response
     */
    public function store(CreateLanguageRequest $request)
    {
        $input = $request->all();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->languageRepository->model());
        try {
            $language = $this->languageRepository->create($input);
            $language->customFieldsValues()->createMany(getCustomFieldsValues($customFields, $request));
            if (isset($input['image']) && $input['image']) {
                $cacheUpload = $this->uploadRepository->getByUuid($input['image']);
                $mediaItem = $cacheUpload->getMedia('image')->first();
                $mediaItem->copy($language, 'image');
            }
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }

        Flash::success(__('lang.saved_successfully', ['operator' => __('lang.language')]));

        return redirect(route('language.index'));
    }

    /**
     * Display the specified Language.
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $language = $this->languageRepository->findWithoutFail($id);

        if (empty($language)) {
            Flash::error('Language not found');

            return redirect(route('language.index'));
        }

        return view('language.show')->with('language', $language);
    }

    /**
     * Show the form for editing the specified Language.
     *
     * @param int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $language = $this->languageRepository->findWithoutFail($id);
        $parentLanguage = $this->languageRepository->pluck('name', 'id')->prepend(__('lang.language_parent_id_placeholder'), '');

        if (empty($language)) {
            Flash::error(__('lang.not_found', ['operator' => __('lang.language')]));

            return redirect(route('language.index'));
        }
        $customFieldsValues = $language->customFieldsValues()->with('customField')->get();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->languageRepository->model());
        $hasCustomField = in_array($this->languageRepository->model(), setting('custom_field_models', []));
        if ($hasCustomField) {
            $html = generateCustomField($customFields, $customFieldsValues);
        }

        return view('language.edit')->with('language', $language)->with("customFields", isset($html) ? $html : false)->with("parentLanguage", $parentLanguage);
    }

    /**
     * Update the specified Language in storage.
     *
     * @param int $id
     * @param UpdateLanguageRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateLanguageRequest $request)
    {
        $language = $this->languageRepository->findWithoutFail($id);

        if (empty($language)) {
            Flash::error('Language not found');
            return redirect(route('language.index'));
        }
        $input = $request->all();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->languageRepository->model());
        try {
            $language = $this->languageRepository->update($input, $id);

            if (isset($input['image']) && $input['image']) {
                $cacheUpload = $this->uploadRepository->getByUuid($input['image']);
                $mediaItem = $cacheUpload->getMedia('image')->first();
                $mediaItem->copy($language, 'image');
            }
            foreach (getCustomFieldsValues($customFields, $request) as $value) {
                $language->customFieldsValues()
                    ->updateOrCreate(['custom_field_id' => $value['custom_field_id']], $value);
            }
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }

        Flash::success(__('lang.updated_successfully', ['operator' => __('lang.language')]));

        return redirect(route('language.index'));
    }

    /**
     * Remove the specified Language from storage.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $language = $this->languageRepository->findWithoutFail($id);

        if (empty($language)) {
            Flash::error('Language not found');

            return redirect(route('language.index'));
        }

        $this->languageRepository->delete($id);

        Flash::success(__('lang.deleted_successfully', ['operator' => __('lang.language')]));

        return redirect(route('language.index'));
    }

    /**
     * Remove Media of Language
     * @param Request $request
     */
    public function removeMedia(Request $request)
    {
        $input = $request->all();
        $language = $this->languageRepository->findWithoutFail($input['id']);
        try {
            if ($language->hasMedia($input['collection'])) {
                $language->getFirstMedia($input['collection'])->delete();
            }
        } catch (Exception $e) {
            Log::error($e->getMessage());
        }
    }
}